// Copyright (C) 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#include <fcntl.h>
#include <jsf_helpers.h>

#include <Sriov.h>
#include <Logging.h>

json Sriov::Invoke(const json& Parameters, bool Iteration) {
  if (Iteration) {
    return {};
  }

  const auto    bdf            = m_device.GetBdf();
  const auto    pcie_pf_device = "/sys/bus/pci/devices/" + bdf;
  const auto    sriov_pf_path  = pcie_pf_device + "/sriov_numvfs";
  std::ifstream sriov_pf_file(sriov_pf_path);
  int           val_sriov_numvfs = 0;
  sriov_pf_file >> val_sriov_numvfs;

  if (Parameters.is_null()) {
    return val_sriov_numvfs;
  }
  if (Parameters.contains("value")) {
    unsigned char val_params = Parameters["value"];

    auto fd_pf_dev = open(sriov_pf_path.c_str(), O_WRONLY);
    if (fd_pf_dev < 0) {
      logFatal("sriov_numvfs: file Not opened");
      m_device.fail();
    }

    if (val_sriov_numvfs == 1 && val_params == 1) {
      close(fd_pf_dev);
      throw std::runtime_error("VF already exists");
    }
    if (val_sriov_numvfs == 0 && val_params == 0) {
      close(fd_pf_dev);
      throw std::runtime_error("VF already destroyed");
    }
    if (val_params > 1) {
      close(fd_pf_dev);
      throw std::runtime_error("invaid value (range: 0-1)");
    }

    if (val_params == 0) {
      auto bdf_vf            = bdf;
      bdf_vf                 = bdf_vf.substr(0, bdf_vf.size() - 1) + "1";
      auto sysfs_driver_path = "/sys/bus/pci/devices/" + bdf_vf + "/driver"; // VM powered-off | unbound
      if (std::filesystem::exists(sysfs_driver_path) == 1) {
        throw std::runtime_error("Shutdown VM and destroy VF device");
      }
    }
    auto ret = write(fd_pf_dev, std::to_string(val_params).c_str(), 1);
    if (ret != 1) {
      logFatal("sriov_numvfs: write failed");
      close(fd_pf_dev);
      m_device.fail();
    }

    if (val_params == 1) { // Unbound VF device
      auto bdf_vf    = bdf;
      bdf_vf         = bdf_vf.substr(0, bdf_vf.size() - 1) + "1";
      auto fd_unbind = open("/sys/bus/pci/drivers/transcoder/unbind", O_WRONLY);
      if (fd_unbind >= 0) {
        auto ret_unbind = write(fd_unbind, bdf_vf.c_str(), sizeof(bdf_vf));
        if (ret_unbind != sizeof(bdf_vf)) {
          logFatal("unbind: write failed");
          close(fd_unbind);
          m_device.fail();
        }
        close(fd_unbind);
      }
    }

    std::ifstream sriov_pf_status(sriov_pf_path);
    sriov_pf_status >> val_sriov_numvfs;
    close(fd_pf_dev);
    return val_sriov_numvfs;
  }
  if (!Parameters.is_null()) {
    throw std::runtime_error("value required / invalid value");
  }
  return {};
}
