// Copyright (C) 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#include "Telemetry.h"

#include <Logging.h>

json DeviceVoltage::GetValue(bool /* Demo */) {
  uint32_t volt = m_device.GetSensorInfo(m_index, m_offset);
  return (((volt * 315) * ((PVT_VREF * 1000) / 1290)) / 1000000);
}

json DeviceTemperature::GetValue(bool /* Demo */) {
  uint32_t tempr = m_device.GetSensorInfo(m_index, m_offset);
  return tempr; // return (((tempr - PVT_DOTP25) * 625) / 10000 + 25);
}

json SensorInfoNode::GetValue(bool /* Demo */) {
  return m_device.GetSensorInfo(m_index, m_offset);
}

json BoardVcNode::GetValue(bool /* Demo */) {
  return m_device.GetBoardVcInfo(m_index, m_offset);
}

json BoardPower::GetValue(bool /* Demo */) {
  uint32_t power_12v_pex = m_device.GetBoardVcInfo(0, m_offset) * m_device.GetBoardVcInfo(3, m_offset);
  uint32_t power_3v_aux  = m_device.GetBoardVcInfo(1, m_offset) * m_device.GetBoardVcInfo(4, m_offset);
  uint32_t power_3v_pex  = m_device.GetBoardVcInfo(2, m_offset) * m_device.GetBoardVcInfo(5, m_offset);
  return (power_12v_pex + power_3v_aux + power_3v_pex) / 1000;
}

json Uptime::Invoke(const json& /* Parameters */, bool /* Iteration */) {
  return m_device.GetUptime(m_offset);
}

json VersionInfo::Invoke(const json& /* Parameters */, bool /* Iteration */) {
  auto ret = m_device.GetVersion(m_index, m_offset);
  return (ret.empty()) ? "ERROR" : ret;
}

json ProductInfoNode::Invoke(const json& /* Parameters */, bool /* Iteration */) {
  auto ret = m_device.GetProductInfo(m_index, m_offset);
  return (ret.empty()) ? "ERROR" : ret;
}

json PcieInfo::Invoke(const json& Parameters, bool Iteration) {
  auto ret = m_device.GetPcieInfo(NodeRawFile::Invoke(Parameters, Iteration));
  return (ret.empty()) ? "ERROR" : ret;
}

json SkuNumber::Invoke(const json& Parameters, bool Iteration) {
  std::string part_no = ProductInfoNode::Invoke(Parameters, Iteration);
  auto        sku_pos = part_no.find_first_of('-') + 1;
  if (sku_pos == std::string::npos) {
    return {};
  }
  auto sku_no = part_no.substr(sku_pos, part_no.find_first_of('\n', sku_pos) - sku_pos);
  return (sku_no.empty()) ? "ERROR" : sku_no;
}

json DdrInfoNode::Invoke(const json& /* Parameters */, bool /* Iteration */) {
  json ddr_bw;
  try {
    ddr_bw["instantaneous"] = m_device.GetDdrInfo(m_index, m_offset);
    // ddr_bw["peak"]          = 0;
  } catch (const std::exception& e) {
    logNotice("%s", e.what());
    ddr_bw["instantaneous"] = "ERROR";
  }

  return ddr_bw;
}

json ErrorCounters::Invoke(const json& /* Parameters */, bool /* Iteration */) {
  json err_counters;
  auto ret                      = m_device.GetErrorCounters(m_index, m_offset);
  err_counters["uncorrectable"] = std::get<0>(ret);
  err_counters["correctable"]   = std::get<1>(ret);
  return err_counters;
}
